package Plugins::Unique;
use Config::General;

my $plugin_name = 'Unique';

my $conf = new Config::General(
    -ConfigFile      => $config::plugins_dir . '/' . $plugin_name . '.conf',
    -InterPolateVars => TRUE,
    -AutoLaunder     => TRUE
);
my %cfg = $conf->getall;

my $VERSION  = 1.0;
my $this_pkg = (caller(0))[0] . '::' . $plugin_name;

$Plugins::Unique::panic      = $cfg{'panic_on_errors'};
@Plugins::Unique::attributes = keys %{ $cfg{'attributes'} };

AOSICFG::write_ws_log('-=- ' . $this_pkg . ': panic on errors=' . $Plugins::Unique::panic) if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . ': getting attributes list: ' . join(", ", @Plugins::Unique::attributes)) if $config::aosi_debug_level > 1;

##################################################################
# Check if an account exists
##################################################################
sub checkEntry {
    my ($base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun,  $msg,         $code,      $out_panic);
    my ($ent_attrs, $attr,        $att,       $vals, $val, $unique, $hash, $result, $dn);
    my ($entries,   $attr_exists, $user_base, $uid);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
    $dn = $entry_in->{'dn'};
    ($uid, $user_base) = AOSICFG::get_uid_base($dn);
    $ent_attrs = $entry_in->{'attrs'};

    foreach $attr (@$ent_attrs) {    # Fetch some attributes
        $att  = $attr->{'att'};
        $vals = $attr->{'val'};
        foreach $val (@$vals) {
            AOSICFG::write_ws_log("|> $this_fun: att=$att") if $config::aosi_debug_level > 3;
            foreach $unique (@Plugins::Unique::attributes) {
                $filter = '(&(' . $att . '=' . $val . ')(!(uid=' . $uid . ')))';

                #(&(mail=aaa@inter1.hr)(!(uid=daaa)))
                if ($unique eq $att) {
                    AOSICFG::write_ws_log("|> $this_fun(looking_for): $att=$val;") if $config::aosi_debug_level > 3;
                    $hash = AOSICFG::ldapGenSearch($this_fun, $user, $pwd, $base_dn, $filter, 'uid', 0, -1, 0);
                    $result = $hash->{'result'};
                    ($entries, $attr_exists) = AOSICFG::xml2entries($result);
                    $ent_attrs = $entries->[0]->{'attrs'};
                    AOSICFG::write_ws_log("|> $this_fun: #entries=" . $#{$ent_attrs}) if $config::aosi_debug_level > 3;
                    if ($#{$ent_attrs} > -1) {
                        $msg .= 'User with UID '
                          . $ent_attrs->[0]->{'val'}->[0]
                          . ' has the same '
                          . $unique . ' ('
                          . $val
                          . ') attribute! ';
                        $code      = -10601;
                        $out_panic = $Plugins::Unique::panic;
                    }
                }
            }
        }
    }
    $msg = $msg eq '' ? 'OK' : $msg;

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
# Check if an account exists
##################################################################
sub beforeAddUser {
    my ($status_hash, $ldap, $base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    ($code, $msg, $out_panic) = checkEntry($base_dn, $entry_in, $user, $pwd);

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
# Check if an account exists
##################################################################
sub afterAddUser {
    my ($status_hash, $entry_in, $out_xml, $user, $pwd) = @_;
    my ($this_fun, $msg,       $code, $out_panic);
    my ($dn,       $ent_attrs, $attr, $search);
    $this_fun  = (caller(0))[3];
    $msg       = 'OK';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg)         if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#  Check if adding an existig value or deleting a nonexisting value
##################################################################
# Same function for all actions
# 1 = beforeAddAttribute
# 2 = beforeModifyAttribute
# 3 = beforeDeleteAttribute
sub beforeChangeAttribute {
    my ($status_hash, $type, $ldap, $base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): tpye=$type;ldap=$ldap;base=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    ($code, $msg, $out_panic) = checkEntry($base_dn, $entry_in, $user, $pwd);

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#  Change attributes (userPassword in AD is in fact unicodePwd)
##################################################################
# Same function for all actions
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
sub afterChangeAttribute {
    my ($status_hash, $type, $entry_in, $out_xml, $user, $pwd) = @_;
    my ($this_fun, $out_panic, $msg);
    $this_fun  = (caller(0))[3];
    $msg       = 'OK';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): tpye=$type;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

1;
